@@ -19,7 +19,7 @@ from scipy import sparse as sp
 from scipy.linalg import norm
 from sklearn.preprocessing import normalize
 from phenograph import bruteforce_nn # temproarly using phenograph on platforms other than Linux
-
+from umap import umap_ as u
 
 if __name__ == '__main__':
     def main():
@@ -61,8 +61,11 @@ def minus_1_normalize_arcsinh(X, noise_threshold = 1, base = 5):
     temp[np.where(np.all(np.isclose(temp, 0), axis=1))[0]] = 1.0
     return(normalize_arcsinh(temp, base=base))
 
-def preprocessing(X, platform = None):
+def pp(X, platform = None):
     temp = type(X)
+    if temp not in [pd.core.frame.DataFrame, np.ndarray]:
+        if not sp.issparse(X):
+            raise RuntimeError('Input datatype not supported!')
     if platform == 'mass':
         if temp is pd.core.frame.DataFrame:
             X = normalize_arcsinh(X.select_dtypes([np.number]).to_numpy(), base = 5)
@@ -82,7 +85,6 @@ def preprocessing(X, platform = None):
     elif platform == 'scrnaseq':
         import scanpy as sc
         import anndata
-
         if temp is anndata._core.anndata.AnnData:
             X = X.X
         if sp.issparse(X):
@@ -100,7 +102,9 @@ def preprocessing(X, platform = None):
         sc.pp.log1p(data)
         X = data.X[:,data.var.highly_variable]
     else:
-        pass
+        if temp is pd.core.frame.DataFrame:
+            X = X.select_dtypes([np.number]).to_numpy()
+        else: pass
     return(X)
 
 def save_as_ijv(filename, X):
@@ -183,118 +187,7 @@ def build_graph(rows, columns, weights, shape):
     warnings.filterwarnings('default')
     return G
 
-
-
-SMOOTH_K_TOLERANCE = 1e-5
-MIN_K_DIST_SCALE = 1e-3
-NPY_INFINITY = np.inf
-
-def smooth_knn_dist(weights, shape, k, n_iter=64, local_connectivity=1.0, bandwidth=1.0):
-    """Compute a continuous version of the distance to the kth nearest
-    neighbor. That is, this is similar to knn-distance but allows continuous
-    k values rather than requiring an integral k. In essence we are simply
-    computing the distance such that the cardinality of fuzzy set we generate
-    is k.
-
-    Parameters
-    ----------
-    distances: array of shape (n_samples, n_neighbors)
-        Distances to nearest neighbors for each samples. Each row should be a
-        sorted list of distances to a given samples nearest neighbors.
-
-    k: float
-        The number of nearest neighbors to approximate for.
-
-    n_iter: int (optional, default 64)
-        We need to binary search for the correct distance value. This is the
-        max number of iterations to use in such a search.
-
-    local_connectivity: int (optional, default 1)
-        The local connectivity required -- i.e. the number of nearest
-        neighbors that should be assumed to be connected at a local level.
-        The higher this value the more connected the manifold becomes
-        locally. In practice this should be not more than the local intrinsic
-        dimension of the manifold.
-
-    bandwidth: float (optional, default 1)
-        The target bandwidth of the kernel, larger values will produce
-        larger return values.
-
-    Returns
-    -------
-    knn_dist: array of shape (n_samples,)
-        The distance to kth nearest neighbor, as suitably approximated.
-
-    nn_dist: array of shape (n_samples,)
-        The distance to the 1st nearest neighbor for each point.
-    """
-    target = np.log2(k) * bandwidth
-    distances = np.array(weights).reshape(shape)
-    rho = np.zeros(shape[0], dtype=np.float32)
-    result = np.zeros(shape[0], dtype=np.float32)
-
-    mean_distances = np.mean(distances)
-
-    for i in range(shape[0]):
-        lo = 0.0
-        hi = NPY_INFINITY
-        mid = 1.0
-
-        # TODO: This is very inefficient, but will do for now. FIXME
-        ith_distances = distances[i]
-        non_zero_dists = ith_distances[ith_distances > 0.0]
-        if non_zero_dists.shape[0] >= local_connectivity:
-            index = int(np.floor(local_connectivity))
-            interpolation = local_connectivity - index
-            if index > 0:
-                rho[i] = non_zero_dists[index - 1]
-                if interpolation > SMOOTH_K_TOLERANCE:
-                    rho[i] += interpolation * (
-                        non_zero_dists[index] - non_zero_dists[index - 1]
-                    )
-            else:
-                rho[i] = interpolation * non_zero_dists[0]
-        elif non_zero_dists.shape[0] > 0:
-            rho[i] = np.max(non_zero_dists)
-
-        for n in range(n_iter):
-
-            psum = 0.0
-            for j in range(1, distances.shape[1]):
-                d = distances[i, j] - rho[i]
-                if d > 0:
-                    psum += np.exp(-(d / mid))
-                else:
-                    psum += 1.0
-
-            if np.fabs(psum - target) < SMOOTH_K_TOLERANCE:
-                break
-
-            if psum > target:
-                hi = mid
-                mid = (lo + hi) / 2.0
-            else:
-                lo = mid
-                if hi == NPY_INFINITY:
-                    mid *= 2
-                else:
-                    mid = (lo + hi) / 2.0
-
-        result[i] = mid
-
-        # TODO: This is very inefficient, but will do for now. FIXME
-        if rho[i] > 0.0:
-            mean_ith_distances = np.mean(ith_distances)
-            if result[i] < MIN_K_DIST_SCALE * mean_ith_distances:
-                result[i] = MIN_K_DIST_SCALE * mean_ith_distances
-        else:
-            if result[i] < MIN_K_DIST_SCALE * mean_distances:
-                result[i] = MIN_K_DIST_SCALE * mean_distances
-
-    return result, rho
-
-
-def build_graph_umap(rows, columns, weights, shape, nbr_num, set_op_mix_ratio=1.0,):
+def build_graph_umap(rows, columns, weights, shape, nbr_num, set_op_mix_ratio=1.0, local_connectivity = 1.0):
     """
     Build a igraph.Graph from the pruned graph
 
@@ -314,12 +207,16 @@ def build_graph_umap(rows, columns, weights, shape, nbr_num, set_op_mix_ratio=1.
     igraph.Graph
         converted graph to be the input of the Leiden algorithm
     """
-    from umap import umap_ as u
     
     warnings.filterwarnings('ignore')
-    distances = np.array(weights).reshape(shape[0], nbr_num)
+    distances = np.array(weights).reshape(shape[0], nbr_num).astype(np.float32)
     indices = np.array(columns).reshape(shape[0], nbr_num)
-    sigmas, rhos = u.smooth_knn_dist(distances, nbr_num)
+    sigmas, rhos = u.smooth_knn_dist(
+        distances,
+        float(nbr_num),
+        local_connectivity=float(local_connectivity),
+    )
+    #sigmas, rhos = ct_smooth_knn_dist(distances, nbr_num)
     r, c, v, d = u.compute_membership_strengths(indices, distances, sigmas, rhos, return_dists = False)
     
     result = sp.coo_matrix((v, (r, c)), shape=shape)
@@ -336,7 +233,6 @@ def build_graph_umap(rows, columns, weights, shape, nbr_num, set_op_mix_ratio=1.
     sources, targets = result.nonzero()
     edgelist = list(zip(sources.tolist(), targets.tolist()))
     G = ig.Graph(edges = edgelist, edge_attrs = {'weight': result.data.tolist()}) 
-    #warnings.filterwarnings('default')
     return G
 
 def build_tani_graph(clu_file, data, local = False, method = 'tani'):
@@ -406,7 +302,7 @@ def la_find_partition(**kwargs):
 
 ###
 
-def find_communities(X, pp_method = None, method = 'tani', nbr_num = 10, n_threads = None, verbose = False, keep_temp_files = False, temp_name = '', partition_type = la.RBConfigurationVertexPartition, resolution = 0.8, seed = None, local = False): 
+def find_communities(X, pp_method = None, method = 'tani', nbr_num = 10, n_threads = None, verbose = False, keep_temp_files = False, temp_name = '', partition_type = la.RBConfigurationVertexPartition, resolution = 0.8, seed = None, local = False, **kwargs): 
     """_summary_
 
     Parameters
@@ -442,7 +338,7 @@ def find_communities(X, pp_method = None, method = 'tani', nbr_num = 10, n_threa
     RuntimeError
         System platform not supported
     """
-    X = preprocessing(X, platform = pp_method)
+    X = pp(X, platform = pp_method)
 
     tic = time.time()
     
@@ -570,8 +466,8 @@ def map_jaccard(idx):
         jaccard_values = pool.starmap(calc_jaccard, zip(range(n), repeat(idx)))
     return jaccard_values
 
-def bc(X, nbr_num = 10, seed = None, metric = "cosine", local = False, method = 'tani', pp_method = None):
-    X = preprocessing(X, platform = pp_method)
+def bc(X, nbr_num = 10, seed = None, metric = "cosine", local = False, method = 'tani', pp_method = None, **kwargs):
+    X = pp(X, platform = pp_method)
     k = nbr_num
     d, idx = bruteforce_nn.knnsearch(X, k + 1, metric)
     idx = idx[:,1:]
@@ -597,9 +493,9 @@ def bc(X, nbr_num = 10, seed = None, metric = "cosine", local = False, method =
     return(communities)
 
 
-def clustering(X, nbr_num = 10, seed = None, local = False, method =  'tani', pp_method = None):
+def clustering(X, nbr_num = 10, seed = None, local = False, method =  'tani', pp_method = None, **kwargs):
     if sys.platform.startswith("linux"):
-        return(find_communities(X, nbr_num=nbr_num, seed = seed, local = local, method = method, pp_method = pp_method))
+        return(find_communities(X, nbr_num=nbr_num, seed = seed, local = local, method = method, pp_method = pp_method, **kwargs))
     else:
-        X = preprocessing(X)
-        return(bc(X, nbr_num= nbr_num, seed = seed, local = local, method = method, pp_method = pp_method))
\ No newline at end of file
+        X = pp(X)
+        return(bc(X, nbr_num= nbr_num, seed = seed, local = local, method = method, pp_method = pp_method, **kwargs))
\ No newline at end of file
